/*
  This Arduino code example exercises two HDC3020 devices (Digital %RH and Temperature sensor).
  The human interface to this firmware is done via UART TXD, using a terminal program such as Teraterm, SerialPlot or Hyperterminal.

  Datasheets:
  -------------------------------------------------
  HDC3020:    https://www.ti.com/lit/ds/symlink/hdc3020.pdf

  User Guides:
  -------------------------------------------------
  MSP-EXP430F5529LP: https://www.ti.com/lit/ug/slau533d/slau533d.pdf
  MEGA2560: https://store-usa.arduino.cc/products/arduino-mega-2560-rev3?selectedStore=us  (used for testing this example)

  Product Pages:
  -------------------------------------------------
  HDC3020:          https://www.ti.com/product/HDC3020

  This code example exercises up to four HDC3020 devices sequentially at 100kHz over I2C bus addresses listed below - two devices are exercised by default here, on
  0x44 and 0x45. The other two can be added by commenting he code for 0x46 and 0x47 back in, and adjusting the postion of the last ln on the prints 

  I2C Addresses:
  ---------------------------------------------------
  I2C Addresses (8 bit ==> 7 bit ==> decimal (for Energia/Arduino))
    -------------
                 8 bit    7 bit
  HDC3020 #1 ==> 0x88 ==> 0x44
  HDC3020 #2 ==> 0x8A ==> 0x45
  HDC3020 #3 ==> 0x8C ==> 0x46
  HDC3020 #4 ==> 0x8E ==> 0x47
    -------------
  ADVANCED FEATURE USAGE:
  N/A

  HARDWARE CONNECTIONS / PIN USAGE:
  -------------------------------------------------
                               3.3V 3.3V
                               /|\  /|\
              HDC3020 %RH/T    4.7k 4.7k     MEGA2560
              I2C responder     |    |    MCU I2C controller
            -----------------   |    |   -------------------
           |    ADDR 0x44    |
           |V+            SDA|<-|----+->|P20/UCB1SDA        |
         |-|ADDR0         SCL|<-+------>|P21/UCB1SCL        |
         |-|ADDR1            |          |                   |
         |-|GND              |          |                   |
         |  -----------------            -------------------
         -
              HDC3020 %RH/T
              I2C responder     |    |
            -----------------   |    |
           |    ADDR 0x45    |
         |-|V+            SDA|<-|----+
         |-|ADDR0         SCL|<-+-----
           |                 |
         |-|ADDR1            |
         |-|GND              |
         |  -----------------
         -

              HDC3020 %RH/T
              I2C responder     |    |
            -----------------   |    |
           |    ADDR 0x46    |
         |-|V+            SDA|<-|----+
         |-|ADDR1         SCL|<-+-----
           |                 |
         |-|ADDR0            |
         |-|GND              |
         |  -----------------
         -

              HDC3020 %RH/T
              I2C responder     |    |
            -----------------   |    |
           |    ADDR 0x47    |
         |-|V+            SDA|<-|----+
         |-|ADDR0         SCL|<-+-----
         |-|ADDR1            |
           |                 |
          |-|GND             |
          |  ----------------
          -

  UART OUTPUT (@115.2kbps, 8N1)
  ------------
  This code puts out comma separated calculated results on UART of the MEGA2560 (TXD pin of the board) for logging purposes (using Teraterm or similar) into .csv file, for use in MSExcel -
  for example:

,HDC302x 2 I2C Address Example
,HDC3 ADDR44 %RH ,  HDC3 ADDR44 TEMP(C), HDC3 ADDR45 %RH,  HDC3 ADDR45 TEMP(C)
,    27.3228     ,        22.8744      ,    30.6462     ,        23.1949
,    27.3289     ,        22.8744      ,    30.6600     ,        23.1762
,    27.3350     ,        22.8531      ,    30.6737     ,        23.1762
,    27.3365     ,        22.8744      ,    30.6966     ,        23.1762
  
  JOSH WYATT
  TEXAS INSTRUMENTS INC.
  12500 TI BLVD.
  DALLAS, TEXAS 75243
  TEMPERATURE AND HUMIDITY SENSOR PRODUCTS
  APPLICATIONS TEAM
  © 2023
*/
#include <Wire.h>

void setup() {
  // initialize serial communication:
  Serial.begin(115200);
  // join i2c bus
  Wire.begin();
  delay(100);
  Serial.print(",");
  Serial.println("HDC302x two I2C Address Example");
  Serial.print(",");
  Serial.println("HDC3 ADDR44 %RH ,  HDC3 ADDR44 TEMP(C), HDC3 ADDR45 %RH,  HDC3 ADDR45 TEMP(C)");  // Print board data logfile column headers
}

void loop() {
  // HDC3x variables that will change:
  float HDC3x_hum_percentage;
  float HDC3x_temp_celcius;
  float HDC3x_1_temp_celcius;
  float HDC3x_2_temp_celcius;
  float HDC3x_3_temp_celcius;
  float HDC3x_4_temp_celcius;
  float HDC3x_1_hum_percentage;
  float HDC3x_2_hum_percentage;
  float HDC3x_3_hum_percentage;
  float HDC3x_4_hum_percentage;

  uint16_t HDC3x_hum_MSB;
  uint16_t HDC3x_hum_DEC;
  uint16_t HDC3x_temp_MSB;
  uint16_t HDC3x_temp_DEC;
  uint16_t HDC3x_temp;

  uint8_t iBuff_HDC3x[4];

  // use the sensors:
  //HDC3 @ address 0x44//
  Wire.beginTransmission(0x44);  // transmit to HDC3x device address 0x44
  Wire.write(0x24);              // 0x24 //read MSB - no clock stretching
  Wire.write(0x00);              // 0x00 //read LSB - HIGH REPEATABILITY
  Wire.endTransmission();        // stop transmitting
  delay(15);                     // delay for conversions
  Wire.requestFrom(0x44, 6, 1);  // request 6 bytes from HDC3x device, device serial number
  // math and UART output here
  while (Wire.available()) {
    Wire.readBytes(iBuff_HDC3x, 6);
    HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                              // shift left
    HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                 // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
    HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;  // do math according to the HDC3x datasheet
    Serial.print(",    ");
    HDC3x_1_hum_percentage = HDC3x_hum_percentage - 1.20;  //unit 1
    Serial.print(HDC3x_1_hum_percentage, 4);        // print out %RH to terminal
    Serial.print("     ,        ");
    HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                               // shift left
    HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                   // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
    HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;  // do math according to the HDC3x datasheet
    HDC3x_1_temp_celcius = HDC3x_temp_celcius;
    Serial.print(HDC3x_1_temp_celcius, 4);
  }
  delay(1);  // delay for MCU
  //HDC3 @ address 0x45//
  Wire.beginTransmission(0x45);  // transmit to HDC3x device address 0x45
  Wire.write(0x24);              // 0x24 //read MSB - no clock stretching
  Wire.write(0x00);              // 0x00 //read LSB - HIGH REPEATABILITY
  Wire.endTransmission();        // stop transmitting
  delay(15);                     // delay for conversions
  Wire.requestFrom(0x45, 6, 1);  // request 6 bytes from HDC3x device, device serial number
  // math and UART output here
  while (Wire.available()) {
    Wire.readBytes(iBuff_HDC3x, 6);
    HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                              // shift left
    HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                 // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
    HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;  // do math according to the HDC3x datasheet
    Serial.print("      ,    ");
    HDC3x_2_hum_percentage = HDC3x_hum_percentage - 5.00;  //unit 2
    Serial.print(HDC3x_2_hum_percentage, 4);        // print out %RH to terminal
    Serial.print("     ,        ");
    HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                               // shift left
    HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                   // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
    HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;  // do math according to the HDC3x datasheet
    HDC3x_2_temp_celcius = HDC3x_temp_celcius;
    Serial.println(HDC3x_2_temp_celcius, 4);
  }
  //          delay(1);                                                                                     // delay for MCU
  //          //HDC3 @ address 0x46//
  //          Wire.beginTransmission(0x46);                                                                 // transmit to HDC3x device address 0x46
  //          Wire.write(0x24);                                                                             // 0x24 //read MSB - no clock stretching
  //          Wire.write(0x00);                                                                             // 0x00 //read LSB - HIGH REPEATABILITY
  //          Wire.endTransmission();                                                                       // stop transmitting
  //          delay(15);                                                                                    // delay for conversions
  //          Wire.requestFrom(0x46, 6, 1);                                                                 // request 6 bytes from HDC3x device, device serial number
  //         // math and UART output here
  //          while (Wire.available())
  //          {
  //            Wire.readBytes(iBuff_HDC3x, 6);
  //            HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                                                        // shift left
  //            HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                                           // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
  //            HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;                            // do math according to the HDC3x datasheet
  //            Serial.print("      ,    ");
  //            HDC3x_3_hum_percentage = HDC3x_hum_percentage;
  //            Serial.print(HDC3x_3_hum_percentage, 4);                                                      // print out %RH to terminal
  //            Serial.print("     ,        ");
  //            HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                                                     // shift left
  //            HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                                         // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
  //            HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;                        // do math according to the HDC3x datasheet
  //            HDC3x_3_temp_celcius = HDC3x_temp_celcius;
  //            Serial.print(HDC3x_3_temp_celcius, 4);
  //          }
  //          delay(1);                                                                                     //delay for MCU
  //          //HDC3 @ address 0x47//
  //          Wire.beginTransmission(0x47);                                                                 // transmit to HDC3x device address 0x47
  //          Wire.write(0x24);                                                                             // 0x24 //read MSB - no clock stretching
  //          Wire.write(0x00);                                                                             // 0x00 //read LSB - HIGH REPEATABILITY
  //          Wire.endTransmission();                                                                       // stop transmitting
  //          delay(15);                                                                                    // delay for conversions
  //          Wire.requestFrom(0x47, 6, 1);                                                                 // request 6 bytes from HDC3x device, device serial number
  //          // math and UART output here
  //          while (Wire.available())
  //          {
  //            Wire.readBytes(iBuff_HDC3x, 6);
  //            HDC3x_hum_MSB = iBuff_HDC3x[3] << 8;                                                        // shift left
  //            HDC3x_hum_DEC = (HDC3x_hum_MSB + iBuff_HDC3x[4]);                                           // get value for calculation, made from iBuff index array values 0 and 1 for humidity reading.
  //            HDC3x_hum_percentage = (((float)(HDC3x_hum_DEC)) / 65535) * 100;                            // do math according to the HDC3x datasheet
  //            Serial.print("      ,    ");
  //            HDC3x_4_hum_percentage = HDC3x_hum_percentage;
  //            Serial.print(HDC3x_4_hum_percentage, 4);                                                      // print out %RH to terminal
  //            Serial.print("     ,        ");
  //            HDC3x_temp_MSB = (iBuff_HDC3x[0] << 8);                                                     // shift left
  //            HDC3x_temp_DEC = (HDC3x_temp_MSB + iBuff_HDC3x[1]);                                         // get value for calculation, made from iBuff index array values 0 and 1 for temp reading.
  //            HDC3x_temp_celcius = ((float)(HDC3x_temp_DEC) / 65535) * (175) - 45;                        // do math according to the HDC3x datasheet
  //            HDC3x_4_temp_celcius = HDC3x_temp_celcius;
  //            Serial.print(HDC3x_4_temp_celcius, 4);
  //          }
  // delay(1000);  // 1 sec loop delay
  delay(1);  // 1mSec loop delay
}
